var __require = /* @__PURE__ */ ((x) => typeof require !== "undefined" ? require : typeof Proxy !== "undefined" ? new Proxy(x, {
  get: (a, b) => (typeof require !== "undefined" ? require : a)[b]
}) : x)(function(x) {
  if (typeof require !== "undefined")
    return require.apply(this, arguments);
  throw Error('Dynamic require of "' + x + '" is not supported');
});

// node_modules/tsup/assets/esm_shims.js
import { fileURLToPath } from "url";
import path from "path";
var getFilename = () => fileURLToPath(import.meta.url);
var getDirname = () => path.dirname(getFilename());
var __dirname = /* @__PURE__ */ getDirname();

// src/Controllers/registerRoutes.ts
import { Router } from "express";
var router = Router();
function registerRoutes(routes, basePath) {
  if (Array.isArray(routes)) {
    for (const route of routes) {
      const PATH = !basePath ? parsePath(`/${route.path}`) : parsePath(`/${basePath}/${route.path}`);
      const entries = Object.entries(route);
      let middleware = null;
      for (let [keys, functions] of entries) {
        switch (keys) {
          case "middleware":
            if (functions)
              middleware = functions;
            break;
          case "get":
          case "post":
          case "put":
          case "delete":
            if (Array.isArray(functions)) {
              functions = middlewareParse(keys, middleware, functions);
              router[keys](PATH, ...functions);
            } else {
              functions = middlewareParse(keys, middleware, functions);
              router[keys](PATH, functions);
            }
            break;
          case "children":
            registerRoutes(functions, PATH);
            break;
        }
      }
    }
  } else {
    const PATH = !basePath ? parsePath(`/${routes.path}`) : parsePath(`/${basePath}/${routes.path}`);
    const entries = Object.entries(routes);
    let middleware = null;
    for (let [keys, functions] of entries) {
      switch (keys) {
        case "middleware":
          if (functions)
            middleware = functions;
          break;
        case "get":
        case "post":
        case "put":
        case "delete":
          if (Array.isArray(functions)) {
            functions = middlewareParse(keys, middleware, functions);
            router[keys](PATH, ...functions);
          } else {
            functions = middlewareParse(keys, middleware, functions);
            router[keys](PATH, functions);
          }
          break;
        case "children":
          registerRoutes(functions, PATH);
          break;
      }
    }
  }
  return router;
}
function parsePath(path4) {
  return path4.replace(new RegExp("//", "g"), "/");
}
function middlewareParse(keys, middleware, func) {
  if (Array.isArray(func)) {
    if (middleware) {
      if (Array.isArray(middleware)) {
        return [...middleware, ...func];
      } else if (typeof middleware === "object") {
        if (middleware[keys]) {
          if (Array.isArray(middleware[keys])) {
            return [...[middleware[keys]], ...func];
          } else {
            return [middleware[keys], ...func];
          }
        }
      } else {
        return [middleware, ...func];
      }
    }
  } else {
    if (middleware) {
      if (Array.isArray(middleware)) {
        return [...middleware, func];
      } else if (typeof middleware === "object") {
        if (middleware[keys]) {
          if (Array.isArray(middleware[keys])) {
            return [...[middleware[keys]], func];
          } else {
            return [middleware[keys], func];
          }
        }
      } else {
        return [middleware, func];
      }
    }
  }
  return func;
}

// src/Controllers/createCrud.ts
import path3 from "path";

// src/Utils/functions.ts
import * as fs from "fs";
import path2 from "path";
function findFolder(directory, nameFolder) {
  let pathFound = null;
  function searchInDirectory(directorio) {
    const elements = fs.readdirSync(directorio);
    for (const element of elements) {
      const fullPath = path2.join(directorio, element);
      const stats = fs.statSync(fullPath);
      if (stats.isDirectory()) {
        if (element === nameFolder) {
          pathFound = fullPath;
          break;
        } else {
          searchInDirectory(fullPath);
        }
      }
    }
  }
  searchInDirectory(directory);
  return pathFound;
}
function createIfNotExists(rutaCarpeta) {
  if (!fs.existsSync(rutaCarpeta))
    fs.mkdirSync(rutaCarpeta, { recursive: true });
}
function capitalizeFirstLetter(texto) {
  return texto.charAt(0).toUpperCase() + texto.substring(1);
}

// src/Controllers/createCrud.ts
import * as fs2 from "fs";
function createCrud(table) {
  table = table.replace(/[^\w\s]/gi, "_");
  const className = capitalizeFirstLetter(table);
  const actualPath = path3.resolve();
  const folder = findFolder(actualPath, "API");
  if (!folder)
    throw "'API' folder not found";
  createService(folder, className);
  const instance = createController(folder, className);
  function routed(options) {
    const routing = {
      middleware: options?.middleware,
      path: table,
      children: []
    };
    const children = {
      middleware: options?.middleware,
      path: ":id"
    };
    const pages = {
      path: "pages/:page",
      get: instance["pages"] ?? []
    };
    if (!options?.methods || options.methods.get)
      routing.get = instance["get"] ?? [];
    if (!options?.methods || options.methods.post)
      routing.post = instance["post"] ?? [];
    if (!options?.methods || options.methods.getOne)
      children.get = instance["getOne"] ?? [];
    if (!options?.methods || options.methods.put)
      children.put = instance["updateOne"] ?? [];
    if (!options?.methods || options.methods.delete)
      children.delete = instance["removeOne"] ?? [];
    routing.children?.push(children);
    if (options?.pages)
      routing.children?.push(pages);
    if (options?.children) {
      if (Array.isArray(options.children))
        routing.children?.concat(options.children);
      else
        routing.children?.push(options.children);
    }
    return registerRoutes(routing);
  }
  return { routed };
}
var createController = (folder, className) => {
  if (!fs2.existsSync(`${folder}/Controllers/${className}.controller.ts`)) {
    createIfNotExists(`${folder}/Controllers`);
    const ControllerOriginal = fs2.readFileSync(`${__dirname}/../dist/Template/Controllers/TEMPLATE_NAME.controller.ts`, "utf-8");
    const ControllerModif = ControllerOriginal.replace(new RegExp("TEMPLATE_NAME", "g"), className);
    fs2.writeFileSync(`${folder}/Controllers/${className}.controller.ts`, ControllerModif, "utf-8");
    console.log(`create ${className}`);
  }
  const instance = __require(`${folder}/Controllers/${className}.controller.ts`);
  return instance;
};
var createService = (folder, serviceName) => {
  if (!fs2.existsSync(`${folder}/Services/${serviceName}.service.ts`)) {
    createIfNotExists(`${folder}/Services`);
    const ControllerOriginal = fs2.readFileSync(`${__dirname}/../dist/Template/Services/TEMPLATE_NAME.service.ts`, "utf-8");
    const ControllerModif = ControllerOriginal.replace(new RegExp("TEMPLATE_NAME", "g"), serviceName);
    fs2.writeFileSync(`${folder}/Services/${serviceName}.service.ts`, ControllerModif, "utf-8");
    console.log(`create service: ${serviceName}`);
  }
};

// src/Controllers/createAPI.ts
function createAPI(params) {
  let route;
  if (Array.isArray(params)) {
    route = [];
    for (const param of params) {
      let options = {};
      if (param.middleware)
        options.middleware = param.middleware;
      if (param.pages)
        options.pages = param.pages;
      if (param.methods)
        options.methods = param.methods;
      if (param.children)
        options.children = param.children;
      route.push(createCrud(param.path).routed(options));
    }
  } else {
    let options = {};
    if (params.middleware)
      options.middleware = params.middleware;
    route = createCrud(params.path).routed(params);
  }
  return route;
}
export {
  createAPI,
  createCrud,
  registerRoutes
};
//! NECESITAMOS VALIDR SI EXISTE EN DIST O EXISTE EN TEST
